<?php 
/*
* Plugin Name: Go WebP
* Plugin URI: https://gowebp.com
* Author: <a href="https://increaserev.com" target="_BLANK" > Increase Rev </a>
* Version: 1.0.6
* Description: Using Go WebP,You can optimize your website to it's potential by optimizing all images. Optimizing images will drastically improved your site speed as  images will load faster. 
*/
if ( ! defined( 'ABSPATH' ) ) exit;
@ini_set( 'error_log', ABSPATH.'/log/php_error.log' );
require_once(dirname(__FILE__).'/vendor/autoload.php');
require_once(dirname(__FILE__).'/includes/helper.php');
require_once(dirname(__FILE__).'/gowebp_worker.php');
require_once(dirname(__FILE__).'/store.php');
class GoWEBP_IR extends GOWEBP_WORKER{

    private static $instance;
    private function __construct(){
        add_action('init', [$this, 'gowebp_admin_init'],0);
        add_action( 'admin_enqueue_scripts', [$this, 'ir_mngr_add_style_and_script'] );
        add_action('admin_menu', [$this, 'ir_mngr_menu_pages']);
        add_action('activated_plugin', [$this, 'ir_webp_start']);
        add_filter( "wp_handle_upload",[$this, "ir_handle_upload"]);
        if($this->ir_check_active()){
            // these 3 will run in admin
            add_action("publish_post",[$this, "ir_after_publish"]);
            add_action("publish_page",[$this, "ir_after_publish"]);
        }
    }
    public static function getInstance(){
        if(self::$instance == null){
            self::$instance = new GoWEBP_IR; 
            return self::$instance;
        }
        return self::$instance;
    }

    function ir_mngr_add_style_and_script(){  
        $current_screen = get_current_screen(); 
        $remaining_days = $this->ir_get_remaining_days();
        if( strpos($current_screen->base, 'gowebp') === false && $remaining_days<=15 && !$this->ir_check_feature_pre()){
           add_action('admin_notices',function() use($remaining_days){
               $domain = sanitize_text_field( $_SERVER["HTTP_HOST"] );
            ?>
            <div class='notice notice-info is-dismissible'>
                <h3>Gowebp</h3>
                <p style='font-size:14px'>Your Plugin trial period will expire in <span style='color:red'> <?=$remaining_days ?> days</span>. <a href="https://gowebp.com/#pricing?domain=<?=$domain ?>" target="_blank"> Get GoWebp.</a> </p>
            </div>
            
            <?php
            });
        }else{

            if (isset($_GET['page']) && ($_GET['page'] == 'gowebp/gowebp_option2.php')) { 
                wp_enqueue_style('trial_style',plugin_dir_url( __FILE__) ."lib/css/trial.css", array(), time());
                wp_enqueue_style('buttons_style',plugin_dir_url( __FILE__) ."lib/css/buttons.css", array(), time());
                wp_enqueue_style('setting_style',plugin_dir_url( __FILE__) ."lib/css/setting_style.css", array(), time());
                wp_enqueue_style('form_style',plugin_dir_url( __FILE__) ."lib/css/form.css", array(), time());
                
                wp_enqueue_style( 'myboot', 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.5.2/css/bootstrap.min.css' ); 
                wp_enqueue_script( 'myboot-script', 'https://cdnjs.cloudflare.com/ajax/libs/twitter-bootstrap/4.5.2/js/bootstrap.bundle.min.js', array(), null, true );
                wp_enqueue_script( "swal","//cdn.jsdelivr.net/npm/sweetalert2@11", ["jquery"], null, true);
                wp_enqueue_script( "knobs","https://cdnjs.cloudflare.com/ajax/libs/jQuery-Knob/1.2.13/jquery.knob.min.js", ["jquery"], null, true);
                wp_enqueue_style( 'load-fa', 'https://use.fontawesome.com/releases/v5.5.0/css/all.css');
                wp_enqueue_style( 'google-fonts', 'https://fonts.googleapis.com/css2?family=Raleway&display=swap');
            }
             

        }
    }

    function ir_mngr_menu_pages(){
        if(current_user_can("administrator")){
        add_menu_page('GOWEBP', 'GOWEBP', 'editor', 'gowebp-menu', '', plugin_dir_url( __FILE__) ."lib/img/Gowebpwp.png", 30);
            add_submenu_page('gowebp-menu', 'GOWEBP', 'Settings', 'administrator', 'gowebp/gowebp_option2.php');
        }
        
    }

    function gowebp_admin_init(){  

        if(is_admin()){
            $GLOBALS['progressbar_show']="hide";
            // For Checking and updating status from server (once per day)
            $this->ir_check_once();
            wp_enqueue_script( "jquery-form" );
            // On settings page
            if($this->ir_check_optStarted() && check_theme_switched() && $this->ir_check_theme_img_status()){
                $ir_form_data = get_user_meta( 1 );
                $ir_theme_image_status  = $ir_form_data["ir_theme_image_status"][0] ?? 0;
                $ir_theme_image_status  = $ir_theme_image_status ==1 ? true : false;
                if($ir_theme_image_status)
                    $this->ir_optimize_theme_images();
            }
        }else{
            // Gowebp Hooks
            if($this->ir_check_active()){
                 add_filter("the_content",[$this, "ir_add_lazyload"]);
                add_filter("post_thumbnail_html",[$this, "ir_add_lazyload_thumbnail2"]);
                
                add_action('    ', [$this, "ir_compiler_funcs"],10);
                if($this->ir_check_optStarted()){
                    // add_action('wp_footer', [$this, "ir_compiler_funcs"],10000000000);
                    add_filter('wp_calculate_image_srcset',[$this, 'ir_update_post_images_url']);
                    // add_filter( 'wp_get_attachment_image_src',[$this, 'ir_update_featured_images_url'], 10, 4);
                    // add_filter( 'wp_get_attachment_image_src',[$this, 'ir_set_logo_img_src'], 10,4);
                    
                    
                }
            }

        }
    }
}

    $gowebp_ir = GoWEBP_IR::getInstance();
    remove_action('shutdown','wp_ob_end_flush_all',1);

