<?php

use lsolesen\pel\Pel;
use lsolesen\pel\PelConvert;
use lsolesen\pel\PelDataWindow;
use lsolesen\pel\PelEntryAscii;
use lsolesen\pel\PelExif;
use lsolesen\pel\PelIfd;
use lsolesen\pel\PelJpeg;
use lsolesen\pel\PelTag;
use lsolesen\pel\PelTiff;

class GOWEBP_WORKER{

    // ReOptimize from start
    public static $re_optimization = false;
    protected static $ir_urls = __DIR__.'/includes/gowebp_urls.json';
    protected static $thumbnail_w;
    protected static $thumbnail_h;

    use Helper\Traits\GoWEBP_Revert;
    use Helper\Traits\GoWEBP_Checker;
    use Helper\Traits\GoWEBP_Updater;
    
    public function getAmpStatus(){
        if($this->ir_check_feature_pre()){
            $status = get_user_meta(1,'ir_amp_status',true) ?? 1;
            return $status == 1;
        }
        return false;
    }
    
    // To not change url when amp status is false
    public function dontChangeUrl(){
        if(!$this->ir_check_js_status() && !$this->getAmpStatus()){
            return true;
        }
        return false;
    }

    // Image Optimization Functions Start +++++++

    // For Post Images
    function ir_add_lazyload($content){
        if(empty($content) || !$this->ir_check_js_status()){
            return  $content;
        }

        $ir_form_data = get_user_meta(1);
        $ir_viewport_status = $ir_form_data["ir_viewport_status"][0] ?? 0;
        $ir_viewport_status = $ir_viewport_status == 0? false:true;
        $ir_to_webp = $ir_form_data["ir_to_webp"][0] ?? 1;
        $ir_to_webp = $ir_to_webp==1 ? true: false;
        $shadow_on_post = $ir_form_data["shadow_on_post"][0] ?? 1;
        $shadow_on_post = $shadow_on_post==1 ? true: false;
        $border_on_post = $ir_form_data["border_on_post"][0] ?? 1;
        $border_on_post = $border_on_post==1 ? true: false;

        $ir_progload_status = $ir_form_data["ir_progload_status"][0] ?? 1;
        $ir_progload_status = $ir_progload_status == 1 ? true: false;


        $content = mb_convert_encoding($content, 'HTML-ENTITIES', "UTF-8");

        $dom = new DOMDocument();
        @$dom->loadHTML($content);
        $img = $dom->getElementsByTagName("img");

        foreach($img as $i){
            $old_source = $i->getAttribute("src");
            $old_source2 = $i->getAttribute("srcset");
            $old_source2 = explode(", ",$old_source2);
            if(is_array($old_source2)){
                $old_source2 = $old_source2[0];
                if(strpos($old_source2," ")!==false){
                    $old_source2 = explode(" ",$old_source2);
                    $old_source2 = $old_source2[0];
                }
            }
            $img_name = substr($old_source,strrpos($old_source,"/")+1);
            $ext = pathinfo($img_name, PATHINFO_EXTENSION);
            if($ext == "gif") continue;
            $suffix = "lg";
            [$is_mobile, $is_tablet] = $this->ir_is_mobile_tablet();
            if($is_mobile){
                $suffix = "sm";
            }elseif($is_tablet){
                $suffix = "md";
            }else{                
                $suffix = "lg";
            }
            if($ir_progload_status == true){
                $img_name2 = str_replace("-holder","_".$suffix,$old_source2);
                $i->setAttribute("src",plugin_dir_url( __FILE__) ."lib/img/imgloader.gif");
                $i->setAttribute("srcset","");
                $i->setAttribute("data-original",$old_source2);
                if($this->ir_check_exists($img_name2))
                    $i->setAttribute("data-responsive",$img_name2);
            }
            elseif($ir_viewport_status == true){
                $i->setAttribute("src",plugin_dir_url( __FILE__) ."lib/img/imgloader.gif");
                $i->setAttribute("srcset","");
                $i->setAttribute("data-original",$old_source2);
            }
            // Changes for Post Images 
            if($shadow_on_post==true ){
                $i->setAttribute("gowebp-shadow","on");
            }
            if($border_on_post==true ){
                $i->setAttribute("gowebp-border","on");
            }
            
        }
        
        
        $newHtml = preg_replace('/^<!DOCTYPE.+?>/', '', str_replace( array('<html>', '</html>', '<body>', '</body>'), array('', '', '', ''), $dom->saveHTML()));
        return $newHtml;
    }

    // For Thumbnail and featured Images
    function ir_add_lazyload_thumbnail($content){
        if(empty($content) || !$this->ir_check_js_status()){
            return  $content;
        }
        $ir_form_data = get_user_meta(1);
        $ir_viewport_status = $ir_form_data["ir_viewport_status"][0] ?? 0;
        $ir_viewport_status = $ir_viewport_status == 0? false:true;
        $ir_to_webp = $ir_form_data["ir_to_webp"][0] ?? 1;
        $ir_to_webp = $ir_to_webp==1 ? true: false;
        $shadow_on_featured = $ir_form_data["shadow_on_featured"][0] ?? 1;
        $shadow_on_featured = $shadow_on_featured==1 ? true: false;
        $shadow_on_thumbnail = $ir_form_data["shadow_on_thumbnail"][0] ?? 1;
        $shadow_on_thumbnail = $shadow_on_thumbnail==1 ? true: false;

        $border_on_featured = $ir_form_data["border_on_featured"][0] ?? 1;
        $border_on_featured = $border_on_featured==1 ? true: false;
        $border_on_thumbnail = $ir_form_data["border_on_thumbnail"][0] ?? 1;
        $border_on_thumbnail = $border_on_thumbnail==1 ? true: false;

        $ir_progload_status = $ir_form_data["ir_progload_status"][0] ?? 1;
        $ir_progload_status = $ir_progload_status == 1 ? true: false;


        $content = mb_convert_encoding($content, 'HTML-ENTITIES', "UTF-8");

        $dom = new DOMDocument();
        @$dom->loadHTML($content);
        $img = $dom->getElementsByTagName("img");

        foreach($img as $i){
            
            $old_source = $i->getAttribute("src");
            $img_name = $old_source;
            $index = $this->ir_getExtIndex($img_name);

            $suffix = "lg";
            [$is_mobile, $is_tablet] = $this->ir_is_mobile_tablet();
            // if mobile or if thumbnail image
            if($is_mobile || (!is_single() && !is_singular())){
                $suffix = "sm";
            }elseif($is_tablet){
                $suffix = "md";
            }

            if($ir_progload_status == true){
                $img_name2 = str_replace("-holder","_".$suffix,$img_name);
                $i->setAttribute("src",plugin_dir_url( __FILE__) ."lib/img/imgloader.gif");
                $i->setAttribute("srcset","");
                $i->setAttribute("data-original",$old_source);
                
                if($this->ir_check_exists($img_name2))
                    $i->setAttribute("data-responsive",$img_name2);
            }

            elseif($ir_viewport_status == true){
                $i->setAttribute("src",plugin_dir_url( __FILE__) ."lib/img/imgloader.gif");
                $i->setAttribute("srcset","");
                $i->setAttribute("data-original",$old_source);

            }
            else{
                $i->setAttribute("src",$old_source);
                $i->setAttribute("srcset",$old_source);

            }

            // Changes for featured - thumbnail Shadow
            if(is_single() && is_singular() && $shadow_on_featured==true ){
                // Post Featured Image
                $i->setAttribute("gowebp-shadow","on");
            }elseif($shadow_on_thumbnail==true && !is_single() && !is_singular()){
                // Thumbnails Image 
                $i->setAttribute("gowebp-shadow","on");
            }
            
            // Changes for featured - thumbnail Border
            if(is_single() && is_singular() && $border_on_featured==true ){
                // Post Featured Image
                $i->setAttribute("gowebp-border","on");
            }elseif($border_on_thumbnail==true && !is_single() && !is_singular()){
                // Thumbnails Image 
                $i->setAttribute("gowebp-border","on");
            }
            
        }
        $newHtml = preg_replace('/^<!DOCTYPE.+?>/', '', str_replace( array('<html>', '</html>', '<body>', '</body>'), array('', '', '', ''), $dom->saveHTML()));
        return $newHtml;
    }
    
    
    function ir_check_exists($url, $gowebp=true){
        $server = $gowebp==true ?wp_upload_dir()["basedir"]."/_gowebp_/" : wp_upload_dir()["path"] ;
        $img_name = $gowebp==true ?substr($url, strrpos($url,"/")+1) :substr($url, strrpos($url,"uploads/")+7) ;
        $img_name = $server.$img_name;
        file_put_contents(__DIR__.'/testlog.txt',"testing: $img_name \n", FILE_APPEND);
        if(file_exists($img_name)) return true;
        
        return false;
    }
    
    function ir_check_webpSupport(){
        $HTTP_ACCEPT = sanitize_text_field( $_SERVER["HTTP_ACCEPT"] );
        if(strpos($HTTP_ACCEPT,"image/webp")!==false){
            return true;
        }
        return false;
    }

    function ir_doOptimize($img_path=""){
        
        $limit = get_user_meta(1, "ir_limit_reached",true) ?? 0;
        if($limit==1){
            return;
        }

        if($this->ir_check_active() && $img_path==""){
            // Form Data Start
            $ir_form_data = get_user_meta(1);
            $ir_optimize_level = $ir_form_data["ir_optimize_level"][0] ?? 65 ;
            $ir_optimize_level = (int) $ir_optimize_level;
            $img_to_webp_ir = $ir_form_data["ir_to_webp"][0] ?? 1;
            $img_to_webp_ir = $img_to_webp_ir == 1 ? true : false;
            // Form Data End
            if(!$this->ir_check_feature_pre()){
                $img_to_webp_ir = false;
            }

            $ir_images = $this->ir_get_images_to_optimize_urls($img_to_webp_ir,$ir_optimize_level);
            $total = count($ir_images);
            if($total!=0) update_user_meta(1,"ir_total_opt_prog",$total);
            self::$thumbnail_h = intval(get_option("thumbnail_size_h",150 ));
            self::$thumbnail_w = intval(get_option("thumbnail_size_w",150 ));
            if($total>0){
                $counter = 0;
                foreach($ir_images as $image){
                    $id = $image["id"];
                    $image = $image["url"];

                    

                    $path = substr($image,0,strrpos($image,"/"));
                    $filename = substr($image,strrpos($image,"/")+1);
                    if($this->ir_is_image($filename)){
                        if(file_exists($image)){    
                            $this->ir_compressor($path,$filename);
                            $this->ir_update_url_array($id,$img_to_webp_ir,$ir_optimize_level);
                            $counter++;
                            update_user_meta(1,"ir_done_opt_prog",$counter);
                        }
                    }
                }
                // Re optimize Deactivate
                update_user_meta( 1, "ir_reOptimize", "0");
                self::$re_optimization = false;
                // Re optimize End
                delete_user_meta(1,"ir_total_opt_prog");
                delete_user_meta(1,"ir_done_opt_prog");

            }
        }elseif($this->ir_check_active()){
            if(file_exists($img_path)){
                
                $img_path = str_replace("\\","/", $img_path);
                $path = substr($img_path,0,strrpos($img_path,"/"));
                $filename = substr($img_path,strrpos($img_path,"/")+1);
                // For Testing
                $this->ir_compressor($path,$filename);
            }
        }
    }

    function ir_compressor($path,$name){
        // Create Directory if it does not exists
        if(!is_file(wp_upload_dir()["basedir"]."/_gowebp_")){
        @mkdir(wp_upload_dir()["basedir"]."/_gowebp_");
        }
        $ir_new_path = wp_upload_dir()["basedir"]."/_gowebp_";

        $imgDir = $path;
        $ir_form_data = get_user_meta(1);
        $ir_reset_status = $ir_form_data["ir_reOptimize"][0] ?? 0 ;
        $ir_reset_status = $ir_reset_status == 1 ? true : false;
        $ir_exif_status = $ir_form_data["ir_exif_status"][0] ?? 0 ;
        $ir_exif_status = $ir_exif_status == 1 ? true : false;
        $ir_optimize_level = isset($ir_form_data["ir_optimize_level"][0]) ? $ir_form_data["ir_optimize_level"][0] : 35;
        $ir_optimize_level = (int) $ir_optimize_level;
        $ir_to_webp = $ir_form_data["ir_to_webp"][0] ?? 1;
        $ir_to_webp = $ir_to_webp==1 ? true: false;

        if(!$this->ir_check_feature_pre()){
            $ir_to_webp  = false;
        }
                
            $index = $this->ir_getExtIndex($name);
            $newName = substr_replace($name,"_ir_img",$index,0);
            $holderName = substr_replace($name,"_ir_img-holder",$index,0);
            // For Webp
            if($ir_to_webp){
                $arr = [".jpeg",".jpg",".png",".bmp"];
                $holderName2 = str_replace($arr,".webp",$holderName);
                $name2 = str_replace($arr,".webp",$newName);
                $this->resize_img_ir($imgDir."/".$name, $ir_new_path."/".$holderName2,$ir_optimize_level,50,50,true,$ir_exif_status);
                $this->resize_img_ir($imgDir."/".$name, $ir_new_path."/".$holderName,$ir_optimize_level,50,50,false, $ir_exif_status);
            }else{ // For Web End
                $this->resize_img_ir($imgDir."/".$name, $ir_new_path."/".$holderName,$ir_optimize_level,50,50, false, $ir_exif_status);
            }

    }

    function resize_img_ir($source, $destination, $ir_optimize_level, $width,$height,$webp=false, $ir_exif_status) {
        $info = getimagesize($source);
            if ($info['mime'] == 'image/jpeg'){
            $image = @imagecreatefromjpeg($source);
        } 
        elseif ($info['mime'] == 'image/gif') {
            $image = @imagecreatefromgif($source);
        }
        elseif ($info['mime'] == 'image/png') {
            $image = @imagecreatefrompng($source);
        }
        elseif ($info['mime'] == 'image/webp') {
            $image = @imagecreatefromwebp($source);
        }
        elseif ($info['mime'] == 'image/bmp') {
            $image = @imagecreatefrombmp($source);
        }

        $original_width = imagesx($image);
        $original_height = imagesy($image);

        $img_resized = imagescale($image,$width,$height);
        $ir_form_data = get_user_meta(1);
        $desktop_dimension = $ir_form_data["ir_desktop_dimension"][0] ?? 85;
        $tablet_dimension = $ir_form_data["ir_tablet_dimension"][0] ?? 50;
        $mobile_dimension = $ir_form_data["ir_mobile_dimension"][0] ?? 35;
        $thumbnail_dimension = $ir_form_data["ir_thumbnail_dimension"][0] ?? 20;
        
        $img_resized_lg = imagescale($image,$original_width*($desktop_dimension/100),$original_height*($desktop_dimension/100));
        $img_resized_md = imagescale($image,$original_width*($tablet_dimension/100),$original_height*($tablet_dimension/100));
        $img_resized_sm = imagescale($image,$original_width*($mobile_dimension/100),$original_height*($mobile_dimension/100));
        $img_resized_xs = imagescale($image,self::$thumbnail_w,self::$thumbnail_h);

        $lg_destination = str_replace("-holder","_lg",$destination);
        $md_destination = str_replace("-holder","_md",$destination);
        $sm_destination = str_replace("-holder","_sm",$destination);
        $xs_destination = str_replace("-holder","_xs",$destination);

        if(!$webp){
            if ($info['mime'] == 'image/jpeg'){
                imagejpeg($img_resized, $destination, $ir_optimize_level);
                // For Different Screen Size
                imagejpeg($img_resized_lg, $lg_destination, $ir_optimize_level);
                imagejpeg($img_resized_md, $md_destination, $ir_optimize_level);
                imagejpeg($img_resized_sm, $sm_destination, $ir_optimize_level);
                imagejpeg($img_resized_xs, $xs_destination, $ir_optimize_level);
                
            } 
            elseif ($info['mime'] == 'image/png') {
                if($image){
                    $arr1 = [
                        $img_resized,
                        $img_resized_lg,
                        $img_resized_md,
                        $img_resized_sm,
                        $img_resized_xs
                    ];
                    $arr2 = [
                        $destination,
                        $lg_destination,
                        $md_destination,
                        $sm_destination,
                        $xs_destination
                    ];
                    // foreach($arr as $key => $value){
                    for($i=0; $i<sizeof($arr1); $i++){
                        $width = imagesx($arr1[$i]);
                        $height = imagesy($arr1[$i]);
                        $new_image = imagecreatetruecolor($width,$height);
                        imagealphablending($new_image,false);
                        imagesavealpha($new_image,true);
                        imagecopyresampled($new_image,$arr1[$i],0,0,0,0,$width, $height, $width, $height);
                        imagepng($new_image, $arr2[$i], floor($ir_optimize_level/10.0));
        
                    }
                }
            }
            elseif ($info['mime'] == 'image/webp') {
                imagewebp($img_resized, $destination, 30);
                // For Different Screen Size
                imagewebp($img_resized_lg, $lg_destination, $ir_optimize_level);
                imagewebp($img_resized_md, $md_destination, $ir_optimize_level);
                imagewebp($img_resized_sm, $sm_destination, $ir_optimize_level);
                imagewebp($img_resized_xs, $xs_destination, $ir_optimize_level);
            }
            elseif ($info['mime'] == 'image/bmp') {
                imagebmp($img_resized, $destination, 30);
                // For Different Screen Size
                imagebmp($img_resized_lg, $lg_destination, $ir_optimize_level);
                imagebmp($img_resized_md, $md_destination, $ir_optimize_level);
                imagebmp($img_resized_sm, $sm_destination, $ir_optimize_level);
                imagebmp($img_resized_xs, $xs_destination, $ir_optimize_level);
            }
        }

        
    } 

    // Post Images Optimization Functions End --------- 


    //  Theme Images Optimization

    function ir_images_iterator($path, &$ir_images){
        $dir = new DirectoryIterator($path);
        foreach($dir as $file){
            if(!$file->isDot()){
                if($file->isDir()){
                    $dirPath = $file->getPath()."/".$file->getBaseName();
                    $this->ir_images_iterator($dirPath,$ir_images);
                }else{
                    if(!$this->ir_is_image($file->getFileName())){
                        continue;
                    }
                    $ir_images[] =$file->getPathname();
                }
            }
        }

    }

    function ir_theme_images_compressor($path, $name){

        $ir_form_data = get_user_meta(1);
        $ir_optimize_level = $ir_form_data["ir_optimize_level"][0] == "" ? 65 : $ir_form_data["ir_optimize_level"][0];
        $ir_optimize_level = (int) $ir_optimize_level;
        $this->ir_theme_images_sub_compressor($path."/".$name, $ir_optimize_level);
    }

    //compress image function

    //  Theme Images Optimization End

    

    // Utility Functions Start
    function ir_is_image($path){
        $arr = ["jpg","jpeg","png","gif","bmp","webp"];
        $ext = pathinfo($path,PATHINFO_EXTENSION);
        if(in_array($ext,$arr)){
            return true;
        }
        return false;
    }

    function ir_extToWebp($url){
        $arr = [".jpg",".jpeg",".png",".gif",".bmp"];
        foreach($arr as $a){
            $url = str_replace($a,".webp",$url);
        }
        return $url;
    }

    function ir_getExtIndex($name){
        $ext = pathinfo($name, PATHINFO_EXTENSION);
        $index = strrpos($name,".$ext");
        return $index;
    }

    function ir_insert_img_count($ir_image_count, $overwrite = false){ 
        update_user_meta(1, "ir_optimized_image_count", $ir_image_count);
    }

    function ir_get_opt_img_count(){
        $opt_level = get_user_meta(1,"ir_optimize_level",true);
        $ext = get_user_meta(1,"ir_to_webp",true);
        $ext = $ext==1?'webp':'legacy';
        $data =$this->ir_get_urls_array();
        $count = 0;
        foreach($data as $image){
            if($image["$ext"]==$opt_level) $count++;
        }
        return $count;
    }

    function ir_get_old_opt_count($webp, $level){
        $data= $this->ir_get_urls_array();
        $ext = $webp ? 'webp' : 'legacy';
        $count=0;
        foreach($data as $image){
            if($image["$ext"]==$level) $count++;
        }
        return $count;
    }


    function ir_is_mobile_tablet() {
        include_once ( plugin_dir_path(__FILE__) . '/Mobile_Detect.php');
        $detect = new Mobile_Detect;
        if( $detect->isTablet() ) {
            return [false,true];
        } elseif($detect->isMobile()) {
            return [true,false];
        }else {
            return [false,false];
        }
    }


    function ir_to_webp(){
        $data = get_user_meta(1);
        $theme_optimize_status = $data["ir_features_status"][0] ?? 'trial';
        $webp = $data["ir_to_webp"][0] ?? '1';
        $webp = $webp == '1'? true : false ;
        if($theme_optimize_status == 'trial' || !$webp){
            return false;
        }
        
        return true;
    }
                
    // Hook Functions

    function ir_after_publish($post_id){
        $images = get_attached_media( 'image', $post_id );
        $path = wp_upload_dir()["path"];
        foreach($images as $image){
            $img_path = wp_get_attachment_url($image->ID);
            $img_path = substr($img_path,strrpos($img_path,"/"));
            $img_edited = substr_replace($img_path,"-edited",$this->ir_getExtIndex($img_path),0);
            $img_path = $path.$img_path;
            // For test
            $img_edited = $path.$img_edited;
            $this->ir_insert_single_image_url($img_path);
            @$this->ir_doOptimize($img_path);
            // Checking if image has been edited
            if($this->ir_check_exists($img_edited, false)){
                $this->ir_insert_single_image_url($img_edited);
                @$this->ir_doOptimize($img_edited);
            }

        }
        @update_user_meta(1,"ir_start_optimization",1);
    }

    function ir_handle_upload($upload){
        $image = $upload["url"];
        $path = wp_upload_dir()["basedir"];
        if($this->ir_is_image($image)){
            $img_path = $image;
            $img_path = substr($img_path,strrpos($img_path,"uploads/")+7);
            $img_path = $path.$img_path;
            $this->ir_insert_single_image_url($img_path);
            $this->ir_doOptimize($img_path);
        }

        return $upload;
    }

    function ir_handle_edit($override, $filename){
        $path = wp_upload_dir()["basedir"];
        if($this->ir_is_image($filename)){
            $this->ir_insert_single_image_url($path.date("Y/m")."/".$filename );
        }
        return $override;
    }

    function ir_save_edited_image($image, $attachment_id){
    $file_path = get_attached_file($attachment_id);
    $this->ir_insert_single_image_url($file_path);
    $this->ir_doOptimize();
    return $image;
    }

    function ir_save_cropped_image($context, $attachment_id, $file_path){
        $this->ir_insert_single_image_url($file_path);
        $this->ir_doOptimize();
    }

    function ir_compiler_funcs(){
            $this->remove_all_scripts_ir();
            $this->load_compress_img_ir();
    }

    function ir_update_post_images_url($sources)
    {
    $ir_avatar = get_avatar_url(1);

    $ir_form_data = get_user_meta(1);
    $img_to_webp_ir = $ir_form_data["ir_to_webp"][0] ?? 1;
    $img_to_webp_ir = $img_to_webp_ir == 1? true : false;
    if(!$this->ir_check_feature_pre()){
        $img_to_webp_ir = false;
    }

    $ir_progload_status = $ir_form_data["ir_progload_status"][0] ?? 1;
    $ir_progload_status = $ir_progload_status == 1? true:false;


    $path = wp_upload_dir()["baseurl"];
    
    // changes
    if($this->dontChangeUrl()){
        return $sources;
    }
    
    foreach($sources as &$source) 
    {
    
        if(strpos($source['url'],substr($path,1)) == false){
            continue;
        }
        if($ir_avatar == $source['url']){
                continue;
        }
        if(pathinfo($source['url'], PATHINFO_EXTENSION) == "gif"){
                continue;
        }
        if(!file_exists($source['url'])) 
        {   
            $img_name = substr($source['url'],strrpos($source['url'],"/")+1); 
            $ext = pathinfo($img_name, PATHINFO_EXTENSION);
            $regex = "/-\d\d\dx\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regex2 = "/-\d\d\dx\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regex3 = "/-\d\dx\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regex4 = "/-\d\dx\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regex5 = "/-\d\d\dx\d\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regex6 = "/-\d\d\d\dx\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regex7 = "/-\d\d\d\dx\d\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
            $regexArr = [$regex,$regex2,$regex3,$regex4,$regex5,$regex6,$regex7];
            foreach($regexArr as $reg){
                $img_name = preg_replace($reg,".$ext",$img_name);
            }
            $index = $this->ir_getExtIndex($img_name);
            $img_name = substr_replace($img_name,"_ir_img-holder", $index,0);
            if(!$ir_progload_status || !$this->ir_check_js_status()){
                [$is_mobile, $is_tablet] = $this->ir_is_mobile_tablet();
                if($is_mobile){
                    $img_name = str_replace("-holder","_sm",$img_name);
                }elseif($is_tablet){
                    $img_name = str_replace("-holder","_md",$img_name);
                }else{                
                    $img_name = str_replace("-holder","_lg",$img_name);
                }
            }
            if($img_to_webp_ir && $this->ir_check_webpSupport()){
                $img_name = $this->ir_extToWebp($img_name);
            }
            // Change url if it exists // fallback
            if($this->ir_check_exists($path."/_gowebp_/".$img_name)) $source['url'] = $path."/_gowebp_/".$img_name;
        }
    }
    return $sources;
    }

    function ir_update_featured_images_url($image, $attachment_id, $size, $icon){
    if (empty($attachment_id)) return $image;
    
    if(! is_singular() && 
    ! is_page() && 
    ! is_single() && 
    ! is_archive() && 
    ! is_home() &&
    ! is_front_page()) {
        return $image;
    }
    
    
    // changes
    if($this->dontChangeUrl()){
        return $image;
    }
    
    if (
    (string) intval($attachment_id) != (string) $attachment_id && 
    $attachment_id != ''
    ) {
        $image[0]= $attachment_id;
    }
    
    
    $path = wp_upload_dir()["baseurl"];
    // Testing
    $path2 = wp_upload_dir()["basedir"];
    $ir_form_data = get_user_meta(1);
    $img_to_webp_ir = $ir_form_data["ir_to_webp"][0] ?? 1 ;
    $img_to_webp_ir = $img_to_webp_ir == 1 ? true: false;
    if(!$this->ir_check_feature_pre()){
        $img_to_webp_ir = false;
    }

    $ir_progload_status = $ir_form_data["ir_progload_status"][0] ?? 1;
    $ir_progload_status = $ir_progload_status == 1 ? true: false;

    $newSrc = substr($image[0],strrpos($image[0],"/")+1);

    $ext = pathinfo($newSrc, PATHINFO_EXTENSION);
    if($ext == "gif") return $image;

    $regex = "/-\d\d\dx\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regex2 = "/-\d\d\dx\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regex3 = "/-\d\dx\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regex4 = "/-\d\dx\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regex5 = "/-\d\d\dx\d\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regex6 = "/-\d\d\d\dx\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regex7 = "/-\d\d\d\dx\d\d\d\d\.(jpg|jpeg|png|gif|webp|bmp)$/";
    $regexArr = [$regex,$regex2,$regex3,$regex4,$regex5,$regex6,$regex7];
    foreach($regexArr as $reg){
        $newSrc = preg_replace($reg,".$ext",$newSrc);    
    }
    $index = $this->ir_getExtIndex($newSrc);
    $newSrc = substr_replace($newSrc,"_ir_img-holder",$index,0);

    if(!$ir_progload_status || !$this->ir_check_js_status()){
        [$is_mobile, $is_tablet] = $this->ir_is_mobile_tablet();
        if($is_mobile){
            $newSrc = str_replace("-holder","_sm",$newSrc);
        }elseif($is_tablet){
            $newSrc = str_replace("-holder","_md",$newSrc);
        }else{                
            $newSrc = str_replace("-holder","_lg",$newSrc);
        }
    }

    

    $img_name = $path."/_gowebp_/".$newSrc;
    
    if($img_to_webp_ir && $this->ir_check_feature_pre() && $this->ir_check_webpSupport()){
        $img_name = $this->ir_extToWebp($img_name);
    }
    if($this->ir_check_exists($img_name)){
        $image[0] = $img_name;
        
    }

    return $image;
    }
    // Hook Functions End


    // Load Optimized Images JS Start

    function load_compress_img_ir(){

    $ir_form_data = get_user_meta(1);
    $ir_optimize_status = $ir_form_data["ir_start_optimization"][0] ?? 0;
    $ir_optimize_status = $ir_optimize_status == 1 ? true : false;

    $img_to_webp_ir = $ir_form_data["ir_to_webp"][0] ?? 1;
    $img_to_webp_ir = $img_to_webp_ir == 1 ? true: false ;

    if(!$this->ir_check_feature_pre()){
        $img_to_webp_ir = false; 
    }
    $ir_border_status = $ir_form_data["ir_border_status"][0] ?? 0;
    $ir_border_status = $ir_border_status == 1 ? true: false;

    $ir_viewport_status = $ir_form_data["ir_viewport_status"][0] ?? 0;
    $ir_viewport_status = $ir_viewport_status == 0 ? false : true;

    $ir_progload_status = $ir_form_data["ir_progload_status"][0] ?? 1;
    $ir_progload_status = $ir_progload_status == 1 ? true : false;

    $border_size = isset($ir_form_data["border_size"][0]) ? $ir_form_data["border_size"][0]:"0";
    $border_color = isset($ir_form_data["border_color"][0])  ? $ir_form_data["border_color"][0]:"#000000";
    $border_radius = isset($ir_form_data["border_radius"][0])  ? $ir_form_data["border_radius"][0]:"0";
    // Shadow Image
    $ir_shadow_status = $ir_form_data["ir_image_shadow"][0] ?? 0;
    $ir_shadow_status = $ir_shadow_status == 1 ? true : false;

    $shadow_size = isset($ir_form_data["ir_shadow_size"][0])  ? $ir_form_data["ir_shadow_size"][0]:"10";
    $shadow_color = isset($ir_form_data["ir_shadow_color"][0]) ? $ir_form_data["ir_shadow_color"][0]:"#000000";
    $shadow_opacity = isset($ir_form_data["ir_shadow_opacity"][0]) ? $ir_form_data["ir_shadow_opacity"][0]:"0.5";

    ?>
    <style>
        /* img[data-responsive], */
        img[data-responsive]{
            filter:blur(0.6em);
        }
        img{
            filter: blur(0em);
        }

    </style>
    <script>
        var img = document.getElementsByTagName("img");

        let ir_optimize_status = "<?php echo $ir_optimize_status ?>";
        let ir_to_webp = '<?php echo $img_to_webp_ir ?>';
        let border_status = "<?php echo $ir_border_status ?>";
        let shadow_status = "<?php echo $ir_shadow_status ?>";
        let progload_status = "<?php echo $ir_progload_status ?>";
        let viewport_status = "<?php echo $ir_viewport_status ?>";
        let screen_size = window.innerWidth;
        let avatar_url = "<?php echo get_avatar_url(1) ?>";
        let ir_path = "<?php echo wp_upload_dir()["baseurl"] ?>";
        for(i of img){
            let source = i.src; 
            if(source.indexOf(avatar_url.slice(0,20)) != -1 ){
                continue;
            }

            if(source.indexOf(".gif") != -1){
                continue;
            }

            if(ir_to_webp == true){
                source = source.replace(".jpg",".webp");
                source = source.replace(".jpeg",".webp");
                source = source.replace(".gif",".webp");
                source = source.replace(".png",".webp");
                source = source.replace(".bmp",".webp");
            }
            source = source.slice(source.lastIndexOf("/")+1);
            source = ir_path+"/_gowebp_/"+source;
            let index = source.indexOf(".jpg");
            if(index == -1){
            index = source.indexOf(".jpeg");
            }
            if(index == -1){
            index = source.indexOf(".png");
            }
            if(index == -1){
            index = source.indexOf(".gif");
            }
            if(index == -1){
            index = source.indexOf(".webp");
            }
            if(index == -1){
                index = source.indexOf(".bmp");
            }
            let border_size = "<?php echo $border_size ?>";
            let border_color = "<?php echo $border_color ?>";
            let border_radius = "<?php echo $border_radius ?>";
            if(border_status == "1" && i.hasAttribute("gowebp-border") ){
                i.style.border = `${border_size}px solid`;
                i.style.borderColor = `${border_color}`;
                i.style.borderRadius = `${border_radius}px`;
            }
            
            let shadow_size = "<?php echo $shadow_size ?>";
            let shadow_color = convertToRGB("<?php echo $shadow_color ?>");
            let shadow_opacity = "<?php echo $shadow_opacity ?>";
            if(shadow_status=="1" && i.hasAttribute("gowebp-shadow")){
                if(shadow_opacity>1){
                    shadow_opacity = 0.9;
                }
                i.style.boxShadow = `0 0 ${shadow_size}px rgba(${shadow_color},${shadow_opacity})`;
            }
        }
        // Setting Compressed image url End

        function convertToRGB(hex) {
            var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(hex);
            if(result){
                var r= parseInt(result[1], 16);
                var g= parseInt(result[2], 16);
                var b= parseInt(result[3], 16);
                return r+","+g+","+b;
                }
            }


        
    let lazyImg = document.querySelectorAll("img[data-src]");
    let lazyImg2 = document.querySelectorAll("img[data-original]");
    let divs = document.querySelectorAll("div");
    // Progressive Image Load
    if(progload_status == true){
    const imgOpt = {
        root:null,
        threshold:0.8,
        rootMargin:"0px"
    }
    const imgOpt2 = {
        root:null,
        threshold:0,
        rootMargin:"0px"
    }
    // --here
    let loadImages = (image)=>{
        if(image.hasAttribute('data-responsive')){
            
            image.setAttribute("src",image.getAttribute("data-responsive"));
            src = image.getAttribute("data-responsive");
            if(src!=null) image.setAttribute("srcset",src);
            if(src!=null) image.setAttribute("src",src);
            image.onload = ()=>{
                image.removeAttribute("data-responsive");
                
            }
        }
    }

    let ir_checkDivs = (div)=>{
        bg = div.style.background;
        if(bg.includes(".gif")){
            return;
        }
        if(bg.includes("ir_img-holder")){
            let width = window.innerWidth;
            suffix = "lg";
            if(width<600){
                suffix = "sm";
            }else if(width<992){
                suffix = "md";
            }
            div.style.background = bg.replace("ir_img-holder","ir_img_"+suffix);
        }
    }

    let ir_progload = (img)=>{
        let source = img.getAttribute("data-original");
        img.setAttribute("src",source);
        
    }

    if('IntersectionObserver' in  window && progload_status==true){
        const observer = new IntersectionObserver((items,observer)=>{
            items.forEach((item)=>{
                if(item.isIntersecting){
                    loadImages(item.target);
                    observer.unobserve(item.target);
                }
            });
        },imgOpt);

        lazyImg.forEach((img)=>{
            observer.observe(img);
        })
        lazyImg2.forEach((img)=>{
            observer.observe(img);
        })
        // =======================================
        const observer2 = new IntersectionObserver((items,observer)=>{
            items.forEach((item)=>{
                if(item.isIntersecting){
                    ir_checkDivs(item.target);
                    observer.unobserve(item.target);
                }
            });
        },imgOpt);
        divs.forEach((div)=>{
            observer2.observe(div);
        });

        // =======================================
        const observer3 = new IntersectionObserver((items,observer)=>{
            items.forEach((item)=>{
                // if(item.intersectionRatio>1){
                if(item.isIntersecting){
                    ir_progload(item.target);
                    observer.unobserve(item.target);
                }
            });
        },imgOpt2);
        lazyImg2.forEach((img)=>{
            observer3.observe(img);
        });

    }else{
        lazyImg.forEach((img)=>{
            loadImages(img);
        });
    }
    }

    let ir_change_viewport = (img) =>{
        source = img.getAttribute("data-original");
        img.setAttribute("src",source);
    }

    if(viewport_status == true){
        options = {};
        observer = new IntersectionObserver((items, observer)=>{
            items.forEach((item)=>{
                if(item.intersectionRatio>0.0){
                    ir_change_viewport(item.target);
                    observer.unobserve(item.target);
                }

            });

        },options);

        lazyImg2.forEach((img)=>{
            observer.observe(img);
        });

    }

    // Progressive Load End 
    </script>
    <?php
    }

    // Load Optimized Images JS End

    // Remove JS Start

    function remove_all_scripts_ir(){
        
        if(!is_admin()){
            ?>
            <script>

                let allTheScripts = document.getElementsByTagName("script");
                let allTheIframes = document.getElementsByTagName("iframess");
                const scriptsSrcArr = [];
                const iframesSrcArr = [];
                for(scrSingle of allTheScripts){
                    if(scrSingle.src == "" || scrSingle.src == null || scrSingle.src == "undefined"){
                        continue;
                    }
                    scriptsSrcArr.push(scrSingle.src);
                    scrSingle.src = "";
                    
                }
                for(ifrSingle of allTheIframes){
                    iframesSrcArr.push(ifrSingle.src);
                    ifrSingle.src = "";
                }        
                let loadStatus = false;
                window.addEventListener("click",function(){
                    if(!loadStatus){
                        reloadScriptsOnEvent(allTheScripts,allTheIframes);                
                    }
                    loadStatus = true     
                });
                window.addEventListener("keydown",function(){
                    if(!loadStatus){
                        reloadScriptsOnEvent(allTheScripts,allTheIframes);                
                    }
                    loadStatus = true     
                });
                window.addEventListener("scroll",function(){
                    if(!loadStatus){
                        reloadScriptsOnEvent(allTheScripts,allTheIframes);                
                    }
                    loadStatus = true     
                });

                function reloadScriptsOnEvent(allTheScripts,allTheIframes){
                    i = 0;
                    for(scrSingle of allTheScripts){
                        if(scriptsSrcArr[i] == "" || scriptsSrcArr[i] == null || scriptsSrcArr[i] == "undefined"){
                            continue;
                        }
                        scrSingle.src = scriptsSrcArr[i];
                        i++;
                    }
                        i=0;
                    for(ifrSingle of allTheIframes){
                        if(iframesSrcArr[i] == "" || iframesSrcArr[i] == null || iframesSrcArr[i] == "undefined"){
                            continue;
                        }
                        ifrSingle.src = iframesSrcArr[i];
                        i++;
                    }
                }
            </script>
            <?php
        }
    }

    // Remove JS End

    // Size Calculation Start
    function ir_calculate_all_images_size(){
        // bytes
        $totalsize = 0;
        // Images Array 
        $ir_images = [];
        // Image Counter
        $imageCounter = 0;

        $insert=true;
        $date = date("Y-m-d");
        $last_date = get_user_meta(1,"ir_url_insert_date",true) ?? 0;
        if($date==$last_date){
            $insert=false;
        }
        // $insert=true;
        @update_user_meta(1,"ir_url_insert_date",$date);
        $path = wp_upload_dir()["basedir"];
        $b = strpos($path, 'wp-content/')+11;
        $path = substr($path,0,$b);
        $this->ir_custom_images_iterator($path,$ir_images,$totalsize,$imageCounter);
        // For Custom paths images Start
        if($this->ir_check_feature_pre()){
            $custom = $this->ir_get_custom_path_img();
            $custom_images = $custom["images"];
            $totalsize += $custom["fileSize"];
            $imageCounter += $custom["count"]*4;
            $ir_images = array_merge($ir_images, $custom_images);
            
        }
        // For Custom paths images End
        update_user_meta( 1, "ir_original_img_count",$imageCounter);
        if($insert){
            $this->ir_insert_images_url($ir_images);
        }
        $this->ir_start_optimization();
        $optImageCounter = $this->ir_get_opt_img_count();

        if($this->ir_check_optStarted() && $optImageCounter>$imageCounter){
            $optImageCounter = $imageCounter;
        }
        if($optImageCounter>1000 && !$this->ir_check_feature_pre()){
            $optImageCounter = 1000;
            update_user_meta(1,"ir_limit_reached","1");
            // $this->ir_update_record('deactive');
        }

        if($optImageCounter>$imageCounter)
            $optImageCounter = $imageCounter;

        return [
            "total_size"=>$totalsize,
            "image_count"=>round($imageCounter/4),
            "optimized_image_count"=>round($optImageCounter/4),
            "pre"=>$this->ir_check_feature_pre()
        ];
    }
// Size Calculation End 

// Custom Path Start

function ir_get_custom_path_img(){
    $paths = get_user_meta(1,"ir_custom_img_paths");
    $ir_images = [];
    $size = 0;
    $count = 0;
    foreach($paths as $path){
        $this->ir_custom_images_iterator($path, $ir_images, $size, $count);
    }

    return [
        "images"=>$ir_images,
        "fileSize"=>$size,
        "count"=>$count
    ];
}

function ir_custom_images_iterator($path, &$ir_images, &$size, &$count){
    $dir = new DirectoryIterator($path);
    foreach($dir as $file){
        if(!$file->isDot()){
            if($file->isDir()){
                if(in_array($file->getBaseName(), ['plugins','cache','upgrade','themes','w3tc-config', 'ngg','_gowebp_'])){
                    continue;
                }
                $dirPath = $file->getPath()."/".$file->getBaseName();
                $this->ir_custom_images_iterator($dirPath,$ir_images, $size, $count);
            }else{
                if(!$this->ir_is_image($file->getFileName())){
                    continue;
                }
                $ir_images[$file->getMTime()][] =$file->getPathname();
                $size += $file->getSize();
                $count++;
            }
        }
    }

}

// Custom Path End

// Custom Logo Start
function ir_set_logo_img_src($image, $attachment_id, $size, $icon){
    if (empty($attachment_id)) return $image;
    if (
    (string) intval($attachment_id) != (string) $attachment_id && 
    $attachment_id != ''
    ) {
        $image[0]= $attachment_id;
    }

    $logo = get_theme_mod( 'custom_logo' );
    $a = json_encode($logo);
    $image = wp_get_attachment_image_src( $logo , 'full' );
    $image_url = $image[0];
    $image_width = $image[1];
    $image_height = $image[2];

    return $image;
    }
// Custom Logo End
}